@@ -35,6 +35,9 @@ char *getGlobalLevelTemplateFilename(void);
 
 int getMappedElement(int);
 
+void ExecuteAsThread(SDL_ThreadFunction, char *, void *, char *);
+char *getPasswordJSON(char *);
+
 void LoadLevelFromFilename(struct LevelInfo *, char *);
 void LoadLevel(int);
 void LoadLevelTemplate(int);
@@ -44,6 +47,7 @@ void SaveLevel(int);
 void SaveLevelTemplate(void);
 void SaveNativeLevel(struct LevelInfo *);
 void DumpLevel(struct LevelInfo *);
+void DumpLevels(void);
 boolean SaveLevelChecked(int);
 
 void CopyNativeLevel_RND_to_Native(struct LevelInfo *);
@@ -54,25 +58,39 @@ void LoadTape(int);
 void LoadSolutionTape(int);
 void SaveTapeToFilename(char *);
 void SaveTape(int);
+void SaveScoreTape(int);
 void DumpTape(struct TapeInfo *);
+void DumpTapes(void);
 boolean SaveTapeChecked(int);
 boolean SaveTapeChecked_LevelSolved(int);
 
 void LoadScore(int);
 void SaveScore(int);
 
+void LoadServerScore(int, boolean);
+void SaveServerScore(int, boolean);
+void SaveServerScoreFromFile(int, boolean, char *);
+
+void LoadLocalAndServerScore(int, boolean);
+
 void LoadUserNames(void);
 
 void LoadSetupFromFilename(char *);
-void LoadSetup(void);
-void SaveSetup(void);
+void LoadSetup_Default(void);
+void SaveSetup_Default(void);
 
 void LoadSetup_AutoSetup(void);
 void SaveSetup_AutoSetup(void);
 
+void LoadSetup_ServerSetup(void);
+void SaveSetup_ServerSetup(void);
+
 void LoadSetup_EditorCascade(void);
 void SaveSetup_EditorCascade(void);
 
+void LoadSetup(void);
+void SaveSetup(void);
+
 void SaveSetup_AddGameControllerMapping(char *);
 
 void setHideSetupEntry(void *);
@@ -90,6 +108,7 @@ void LoadHelpTextInfo(void);
 
 void ConvertLevels(void);
 void CreateLevelSketchImages(void);
+void CreateCollectElementImages(void);
 void CreateCustomElementImages(char *);
 
 void FreeGlobalAnimEventInfo(void);
