@@ -114,6 +114,11 @@ typedef struct hashtable     SetupFileHash;
   }								\
 
 
+// sort priorities of special tree entries
+#define LEVELCLASS_TOP			0
+#define LEVELCLASS_PARENT		1
+#define LEVELCLASS_LAST_PLAYED_LEVEL	2
+
 // sort priorities of level series (also used as level series classes)
 #define LEVELCLASS_TUTORIAL_START	10
 #define LEVELCLASS_TUTORIAL_END		99
@@ -227,6 +232,30 @@ typedef struct hashtable     SetupFileHash;
 			 IS_ARTWORKCLASS_LEVEL(n) ? ARTWORKCLASS_LEVEL :      \
 			 ARTWORKCLASS_UNDEFINED)
 
+#define TREE_SORTING_DIR(ti)						\
+	(((ti)->parent_link				   ? 0 :	\
+	  (ti)->in_user_dir				   ? 4 * 200 :	\
+	  (ti)->sort_priority >= LEVELCLASS_CLASSICS_START ? 3 * 200 +	\
+	  (ti)->sort_priority % 100 :					\
+	  (ti)->sort_priority >= LEVELCLASS_TUTORIAL_START ? 2 * 200 +	\
+	  (ti)->sort_priority % 100 :					\
+	  1 * 200) +							\
+	 ((ti)->level_group ? 0 : 100))
+
+#define TREE_COLOR_DIR(ti, active)					\
+	((active)			? FC_YELLOW :			\
+	 TREE_SORTING(ti) / 200 == 4	? FC_GREEN :			\
+	 TREE_SORTING(ti) / 200 == 2	? FC_BLUE :			\
+	 FC_RED)
+
+#define TREE_SORTING(ti)						\
+	(TREE_TYPE_IS_DIR((ti)->type) ? TREE_SORTING_DIR(ti) :		\
+	 (ti)->sort_priority)
+
+#define TREE_COLOR(ti, active)						\
+	(TREE_TYPE_IS_DIR((ti)->type) ? TREE_COLOR_DIR(ti, active) :	\
+	 (ti)->color)
+
 
 char *getUserGraphicsDir(void);
 char *getUserSoundsDir(void);
@@ -235,8 +264,14 @@ char *setLevelArtworkDir(TreeInfo *);
 char *getProgramMainDataPath(char *, char *);
 char *getProgramConfigFilename(char *);
 char *getTapeFilename(int);
+char *getTemporaryTapeFilename(void);
+char *getDefaultSolutionTapeFilename(int);
+char *getSokobanSolutionTapeFilename(int);
 char *getSolutionTapeFilename(int);
 char *getScoreFilename(int);
+char *getScoreCacheFilename(int);
+char *getScoreTapeBasename(char *);
+char *getScoreTapeFilename(char *, int);
 char *getSetupFilename(void);
 char *getDefaultSetupFilename(void);
 char *getEditorSetupFilename(void);
@@ -253,8 +288,14 @@ char *getCustomArtworkConfigFilename(int);
 char *getCustomArtworkLevelConfigFilename(int);
 char *getCustomMusicDirectory(void);
 
+void MarkTapeDirectoryUploadsAsComplete(char *);
+void MarkTapeDirectoryUploadsAsIncomplete(char *);
+boolean CheckTapeDirectoryUploadsComplete(char *);
+
 void InitTapeDirectory(char *);
 void InitScoreDirectory(char *);
+void InitScoreCacheDirectory(char *);
+void InitScoreTapeDirectory(char *, int);
 void InitUserLevelDirectory(char *);
 void InitNetworkLevelDirectory(char *);
 void InitLevelSetupDirectory(char *);
@@ -262,15 +303,18 @@ void InitLevelSetupDirectory(char *);
 TreeInfo *newTreeInfo(void);
 TreeInfo *newTreeInfo_setDefaults(int);
 void pushTreeInfo(TreeInfo **, TreeInfo *);
+void removeTreeInfo(TreeInfo **);
 int numTreeInfo(TreeInfo *);
 boolean validLevelSeries(TreeInfo *);
+TreeInfo *getValidLevelSeries(TreeInfo *, TreeInfo *);
 TreeInfo *getFirstValidTreeInfoEntry(TreeInfo *);
+TreeInfo *getNextValidTreeInfoEntry(TreeInfo *);
 TreeInfo *getTreeInfoFirstGroupEntry(TreeInfo *);
 int numTreeInfoInGroup(TreeInfo *);
 int getPosFromTreeInfo(TreeInfo *);
 TreeInfo *getTreeInfoFromPos(TreeInfo *, int);
 TreeInfo *getTreeInfoFromIdentifier(TreeInfo *, char *);
-void dumpTreeInfo(TreeInfo *, int);
+int dumpTreeInfo(TreeInfo *, int);
 void sortTreeInfoBySortFunction(TreeInfo **,
 				int (*compare_function)(const void *,
 							const void *));
@@ -278,7 +322,6 @@ void sortTreeInfo(TreeInfo **);
 void freeTreeInfo(TreeInfo *);
 
 char *getHomeDir(void);
-char *getCommonDataDir(void);
 char *getPersonalDataDir(void);
 char *getMainUserGameDataDir(void);
 char *getUserGameDataDir(void);
@@ -288,6 +331,7 @@ char *getUserLevelDir(char *);
 char *getNetworkLevelDir(char *);
 char *getCurrentLevelDir(void);
 char *getNewUserLevelSubdir(void);
+char *getTapeDir(char *);
 
 void createDirectory(char *, char *, int);
 void InitMainUserDataDirectory(void);
@@ -321,6 +365,7 @@ char *getSetupValue(int, void *);
 char *getSetupLine(struct TokenInfo *, char *, int);
 
 unsigned int get_hash_from_key(void *);
+int hash_keys_are_equal(void *, void *);
 
 int GetZipFileTreeType(char *);
 char *ExtractZipFileIntoDirectory(char *, char *, int);
@@ -343,6 +388,10 @@ void AddTreeSetToTreeInfo(TreeInfo *, char *, char *, int);
 boolean UpdateUserLevelSet(char *, char *, char *, int);
 boolean CreateUserLevelSet(char *, char *, char *, int, boolean);
 
+void UpdateLastPlayedLevels_TreeInfo(void);
+void StoreLastPlayedLevels(TreeInfo *);
+void RestoreLastPlayedLevels(TreeInfo **);
+
 void LoadLevelSetup_LastSeries(void);
 void SaveLevelSetup_LastSeries(void);
 void SaveLevelSetup_LastSeries_Deactivate(void);
