@@ -22,6 +22,8 @@
 #include "windows.h"
 #elif defined(PLATFORM_ANDROID)
 #include "android.h"
+#elif defined(PLATFORM_EMSCRIPTEN)
+#include "emscripten.h"
 #endif
 
 #include "sdl.h"
@@ -103,6 +105,23 @@
 #define STR_NETWORK_AUTO_DETECT		"auto_detect_network_server"
 #define STR_NETWORK_AUTO_DETECT_SETUP	"(auto detect network server)"
 
+// values for API server settings
+#define API_SERVER_HOSTNAME		"api.artsoft.org"
+#define API_SERVER_PORT			80
+#define API_SERVER_METHOD		"POST"
+#define API_SERVER_URI_ADD		"/api/scores/add"
+#define API_SERVER_URI_GET		"/api/scores/get"
+#define API_SERVER_URI_RENAME		"/api/players/rename"
+#define API_SERVER_URI_RESETUUID	"/api/players/resetuuid"
+
+#if defined(TESTING)
+#undef API_SERVER_HOSTNAME
+#define API_SERVER_HOSTNAME		"api-test.artsoft.org"
+#define TEST_PREFIX			"test."
+#else
+#define TEST_PREFIX			""
+#endif
+
 // values for touch control
 #define TOUCH_CONTROL_OFF		"off"
 #define TOUCH_CONTROL_VIRTUAL_BUTTONS	"virtual_buttons"
@@ -350,6 +369,8 @@
 #define ANIM_STATIC_PANEL	(1 << 13)
 #define ANIM_ALL		(1 << 14)
 #define ANIM_ONCE		(1 << 15)
+#define ANIM_TILED		(1 << 16)
+#define ANIM_RANDOM_STATIC	(1 << 17)
 
 #define ANIM_DEFAULT		ANIM_LOOP
 
@@ -581,6 +602,9 @@
 					 JOY_BUTTON_2 :			 \
 					 JOY_NO_ACTION)
 
+// maximum number of level sets in the level set history
+#define MAX_LEVELDIR_HISTORY	12
+
 // default name for empty highscore entry
 #define EMPTY_PLAYER_NAME	"no name"
 
@@ -602,6 +626,9 @@
 // default value for undefined levelset
 #define UNDEFINED_LEVELSET	"[NONE]"
 
+// default value for undefined password
+#define UNDEFINED_PASSWORD	"[undefined]"
+
 // default value for undefined parameter
 #define ARG_DEFAULT		"[DEFAULT]"
 
@@ -612,18 +639,11 @@
 // default value for off-screen positions
 #define POS_OFFSCREEN		(-1000000)
 
-// definitions for game sub-directories
-#ifndef RO_GAME_DIR
-#define RO_GAME_DIR		"."
+// definitions for game base path and sub-directories
+#ifndef BASE_PATH
+#define BASE_PATH		"."
 #endif
 
-#ifndef RW_GAME_DIR
-#define RW_GAME_DIR		"."
-#endif
-
-#define RO_BASE_PATH		RO_GAME_DIR
-#define RW_BASE_PATH		RW_GAME_DIR
-
 // directory names
 #define GRAPHICS_DIRECTORY	"graphics"
 #define SOUNDS_DIRECTORY	"sounds"
@@ -657,6 +677,7 @@
 #define USERSETUP_FILENAME	"usersetup.conf"
 #define AUTOSETUP_FILENAME	"autosetup.conf"
 #define LEVELSETUP_FILENAME	"levelsetup.conf"
+#define SERVERSETUP_FILENAME	"serversetup.conf"
 #define EDITORSETUP_FILENAME	"editorsetup.conf"
 #define EDITORCASCADE_FILENAME	"editorcascade.conf"
 #define HELPANIM_FILENAME	"helpanim.conf"
@@ -667,6 +688,7 @@
 #define MUSICINFO_FILENAME	"musicinfo.conf"
 #define ARTWORKINFO_CACHE_FILE	"artworkinfo.cache"
 #define LEVELTEMPLATE_FILENAME	"template.level"
+#define UPLOADED_FILENAME	".uploaded"
 #define LEVELFILE_EXTENSION	"level"
 #define TAPEFILE_EXTENSION	"tape"
 #define SCOREFILE_EXTENSION	"score"
@@ -771,6 +793,11 @@
 #define NUM_BASE_TREE_TYPES	4
 #define NUM_TREE_TYPES		6
 
+#define TREE_TYPE_IS_DIR(type)	((type) == TREE_TYPE_GRAPHICS_DIR ||	\
+				 (type) == TREE_TYPE_SOUNDS_DIR ||	\
+				 (type) == TREE_TYPE_MUSIC_DIR ||	\
+				 (type) == TREE_TYPE_LEVEL_DIR)
+
 #define INFOTEXT_UNDEFINED	""
 #define INFOTEXT_GRAPHICS_DIR	"Custom Graphics"
 #define INFOTEXT_SOUNDS_DIR	"Custom Sounds"
@@ -779,6 +806,10 @@
 #define INFOTEXT_LEVEL_NR	"Levels"
 #define INFOTEXT_PLAYER_NAME	"Players & Teams"
 
+#define BACKLINK_TEXT_MAIN	".. (main menu)"
+#define BACKLINK_TEXT_SETUP	".. (setup menu)"
+#define BACKLINK_TEXT_PARENT	".. (parent directory)"
+
 #define TREE_INFOTEXT(t)	((t) == TREE_TYPE_PLAYER_NAME ?		\
 				 INFOTEXT_PLAYER_NAME :			\
 				 (t) == TREE_TYPE_LEVEL_NR ?		\
@@ -793,6 +824,10 @@
 				 INFOTEXT_MUSIC_DIR :			\
 				 INFOTEXT_UNDEFINED)
 
+#define TREE_BACKLINK_TEXT(t)	((t) == TREE_TYPE_LEVEL_DIR ?		\
+				 BACKLINK_TEXT_MAIN :			\
+				 BACKLINK_TEXT_SETUP)
+
 #define TREE_USERDIR(t)		((t) == TREE_TYPE_LEVEL_DIR ?		\
 				 getUserLevelDir(NULL) :		\
 				 (t) == TREE_TYPE_GRAPHICS_DIR ?	\
@@ -992,8 +1027,7 @@ struct ProgramInfo
   void (*exit_message_function)(char *, va_list);
   void (*exit_function)(int);
 
-  boolean global_scores;
-  boolean many_scores_per_name;
+  int api_thread_count;
 
   boolean headless;
 };
@@ -1014,6 +1048,8 @@ struct NetworkInfo
 struct RuntimeInfo
 {
   boolean uses_touch_device;
+
+  boolean use_api_server;
 };
 
 struct OptionInfo
@@ -1021,8 +1057,7 @@ struct OptionInfo
   char *server_host;
   int server_port;
 
-  char *ro_base_directory;
-  char *rw_base_directory;
+  char *base_directory;
   char *level_directory;
   char *graphics_directory;
   char *sounds_directory;
@@ -1031,10 +1066,15 @@ struct OptionInfo
   char *conf_directory;
 
   char *execute_command;
+  char *tape_log_filename;
 
   char *special_flags;
   char *debug_mode;
 
+  char *player_name;
+  char *identifier;
+  char *level_nr;
+
   boolean mytapes;
   boolean serveronly;
   boolean network;
@@ -1294,6 +1334,11 @@ struct SetupAutoSetupInfo
   int editor_zoom_tilesize;
 };
 
+struct SetupLevelSetupInfo
+{
+  char *last_level_series[MAX_LEVELDIR_HISTORY + 1];
+};
+
 struct SetupEditorCascadeInfo
 {
   boolean el_bd;
@@ -1310,6 +1355,7 @@ struct SetupEditorCascadeInfo
   boolean el_steel_chars;
   boolean el_ce;
   boolean el_ge;
+  boolean el_es;
   boolean el_ref;
   boolean el_user;
   boolean el_dynamic;
@@ -1405,6 +1451,8 @@ struct SetupDebugInfo
 struct SetupInfo
 {
   char *player_name;
+  char *player_uuid;
+  int player_version;
 
   boolean multiple_users;
 
@@ -1427,7 +1475,8 @@ struct SetupInfo
   boolean skip_levels;
   boolean increment_levels;
   boolean auto_play_next_level;
-  boolean skip_scores_after_game;
+  boolean count_score_after_game;
+  boolean show_scores_after_game;
   boolean time_limit;
   boolean fullscreen;
   int window_scaling_percent;
@@ -1437,6 +1486,8 @@ struct SetupInfo
   boolean ask_on_escape;
   boolean ask_on_escape_editor;
   boolean ask_on_game_over;
+  boolean ask_on_quit_game;
+  boolean ask_on_quit_program;
   boolean quick_switch;
   boolean input_on_focus;
   boolean prefer_aga_graphics;
@@ -1446,7 +1497,9 @@ struct SetupInfo
   int game_frame_delay;
   boolean sp_show_border_elements;
   boolean small_game_graphics;
-  boolean show_snapshot_buttons;
+  boolean show_load_save_buttons;
+  boolean show_undo_redo_buttons;
+  char *scores_in_highscore_list;
 
   char *graphics_set;
   char *sounds_set;
@@ -1463,7 +1516,18 @@ struct SetupInfo
   int network_player_nr;
   char *network_server_hostname;
 
+  boolean use_api_server;
+  char *api_server_hostname;
+  char *api_server_password;
+  boolean ask_for_uploading_tapes;
+  boolean ask_for_remaining_tapes;
+  boolean provide_uploading_tapes;
+  boolean ask_for_using_api_server;
+  boolean has_remaining_tapes;
+
   struct SetupAutoSetupInfo auto_setup;
+  struct SetupLevelSetupInfo level_setup;
+
   struct SetupEditorInfo editor;
   struct SetupEditorCascadeInfo editor_cascade;
   struct SetupShortcutInfo shortcut;
@@ -1537,6 +1601,7 @@ struct TreeInfo
 
   boolean level_group;	// directory contains more level series directories
   boolean parent_link;	// entry links back to parent directory
+  boolean is_copy;	// this entry is a copy of another entry in the tree
   boolean in_user_dir;	// user defined levels are stored in home directory
   boolean user_defined;	// levels in user directory and marked as "private"
   boolean readonly;	// readonly levels can not be changed with editor
@@ -1833,7 +1898,6 @@ void InitProgramInfo(char *, char *, char *, char *, char *, char *, char *,
 void InitNetworkInfo(boolean, boolean, boolean, char *, int);
 void InitRuntimeInfo(void);
 
-void InitScoresInfo(void);
 void SetWindowTitle(void);
 
 void InitWindowTitleFunction(char *(*window_title_function)(void));
@@ -1922,6 +1986,7 @@ Bitmap *LoadImage(char *);
 Bitmap *LoadCustomImage(char *);
 void ReloadCustomImage(Bitmap *, char *);
 
+Bitmap *ZoomBitmap(Bitmap *, int, int);
 void ReCreateGameTileSizeBitmap(Bitmap **);
 void CreateBitmapWithSmallBitmaps(Bitmap **, int, int);
 void CreateBitmapTextures(Bitmap **);
@@ -1955,4 +2020,7 @@ boolean ReadJoystick(int, int *, int *, boolean *, boolean *);
 boolean CheckJoystickOpened(int);
 void ClearJoystickState(void);
 
+void InitEmscriptenFilesystem(void);
+void SyncEmscriptenFilesystem(void);
+
 #endif // SYSTEM_H
