@@ -312,7 +312,8 @@ static void DrawVideoDisplay_DateTime(unsigned int state, unsigned int value)
       char s[MAX_DATETIME_STRING_SIZE];
       int year2 = value / 10000;
       int year4 = (year2 < 70 ? 2000 + year2 : 1900 + year2);
-      int month_index = (value / 100) % 100;
+      int month_index_raw = (value / 100) % 100;
+      int month_index = month_index_raw % 12;	// prevent invalid index
       int month = month_index + 1;
       int day = value % 100;
 
@@ -470,8 +471,44 @@ void TapeDeactivateDisplayOff(boolean redraw_display)
 // tape logging functions
 // ============================================================================
 
+struct AutoPlayInfo
+{
+  LevelDirTree *leveldir;
+  boolean all_levelsets;
+  int last_level_nr;
+  int level_nr;
+  int num_levels_played;
+  int num_levels_solved;
+  int num_tapes_patched;
+  int num_tape_missing;
+  boolean level_failed[MAX_TAPES_PER_SET];
+  char *tape_filename;
+};
+
 static char tape_patch_info[MAX_OUTPUT_LINESIZE];
 
+static void PrintTapeReplayHeader(struct AutoPlayInfo *autoplay)
+{
+  PrintLine("=", 79);
+
+  if (global.autoplay_mode == AUTOPLAY_MODE_FIX)
+    Print("Automatically fixing level tapes\n");
+  else if (global.autoplay_mode == AUTOPLAY_MODE_UPLOAD)
+    Print("Automatically uploading level tapes\n");
+  else
+    Print("Automatically playing level tapes\n");
+
+  PrintLine("-", 79);
+  Print("Level series identifier: '%s'\n", autoplay->leveldir->identifier);
+  Print("Level series name:       '%s'\n", autoplay->leveldir->name);
+  Print("Level series author:     '%s'\n", autoplay->leveldir->author);
+  Print("Number of levels:        %d\n",   autoplay->leveldir->levels);
+  PrintLine("=", 79);
+  Print("\n");
+
+  DrawInitTextItem(autoplay->leveldir->name);
+}
+
 static void PrintTapeReplayProgress(boolean replay_finished)
 {
   static unsigned int counter_last = -1;
@@ -512,6 +549,79 @@ static void PrintTapeReplayProgress(boolean replay_finished)
   }
 }
 
+static void PrintTapeReplaySummary(struct AutoPlayInfo *autoplay)
+{
+  char *autoplay_status =
+    (autoplay->num_levels_played == autoplay->num_levels_solved &&
+     autoplay->num_levels_played > 0 ? " OK " : "WARN");
+  int autoplay_percent =
+    (autoplay->num_levels_played ?
+     autoplay->num_levels_solved * 100 / autoplay->num_levels_played : 0);
+  int i;
+
+  Print("\n");
+  PrintLine("=", 79);
+  Print("Number of levels played: %d\n", autoplay->num_levels_played);
+  Print("Number of levels solved: %d (%d%%)\n", autoplay->num_levels_solved,
+	(autoplay->num_levels_played ?
+	 autoplay->num_levels_solved * 100 / autoplay->num_levels_played : 0));
+  if (global.autoplay_mode == AUTOPLAY_MODE_FIX)
+    Print("Number of tapes fixed: %d\n", autoplay->num_tapes_patched);
+  PrintLine("-", 79);
+  Print("Summary (for automatic parsing by scripts):\n");
+
+  if (autoplay->tape_filename)
+  {
+    Print("TAPEFILE [%s] '%s', %d, %d, %d",
+	  autoplay_status,
+	  autoplay->leveldir->identifier,
+	  autoplay->last_level_nr,
+	  game.score_final,
+	  game.score_time_final);
+  }
+  else
+  {
+    Print("LEVELDIR [%s] '%s', SOLVED %d/%d (%d%%)",
+	  autoplay_status,
+	  autoplay->leveldir->identifier,
+	  autoplay->num_levels_solved,
+	  autoplay->num_levels_played,
+	  autoplay_percent);
+
+    if (autoplay->num_levels_played != autoplay->num_levels_solved)
+    {
+      Print(", FAILED:");
+      for (i = 0; i < MAX_TAPES_PER_SET; i++)
+	if (autoplay->level_failed[i])
+	  Print(" %03d", i);
+    }
+  }
+
+  Print("\n");
+  PrintLine("=", 79);
+}
+
+static FILE *tape_log_file;
+
+static void OpenTapeLogfile(void)
+{
+  if (!(tape_log_file = fopen(options.tape_log_filename, MODE_WRITE)))
+    Warn("cannot write tape logfile '%s'", options.tape_log_filename);
+}
+
+static void WriteTapeLogfile(byte action[MAX_TAPE_ACTIONS])
+{
+  int i;
+
+  for (i = 0; i < MAX_TAPE_ACTIONS; i++)
+    putFile8Bit(tape_log_file, action[i]);
+}
+
+static void CloseTapeLogfile(void)
+{
+  fclose(tape_log_file);
+}
+
 
 // ============================================================================
 // tape control functions
@@ -540,8 +650,14 @@ void TapeErase(void)
   tape.length_frames = 0;
   tape.length_seconds = 0;
 
+  tape.score_tape_basename[0] = '\0';
+
   if (leveldir_current)
-    setString(&tape.level_identifier, leveldir_current->identifier);
+  {
+    strncpy(tape.level_identifier, leveldir_current->identifier,
+	    MAX_FILENAME_LEN);
+    tape.level_identifier[MAX_FILENAME_LEN] = '\0';
+  }
 
   tape.level_nr = level_nr;
   tape.pos[tape.counter].delay = 0;
@@ -553,6 +669,8 @@ void TapeErase(void)
   tape.game_version = GAME_VERSION_ACTUAL;
   tape.engine_version = level.game_version;
 
+  tape.property_bits = TAPE_PROPERTY_NONE;
+
   TapeSetDateFromNow();
 
   for (i = 0; i < MAX_PLAYERS; i++)
@@ -644,6 +762,8 @@ static void TapeAppendRecording(void)
   // set current delay (for last played move)
   tape.pos[tape.counter].delay = tape.delay_played;
 
+  tape.property_bits |= TAPE_PROPERTY_REPLAYED;
+
   // set current date
   TapeSetDateFromNow();
 
@@ -734,6 +854,12 @@ void TapeRecordAction(byte action_raw[MAX_TAPE_ACTIONS])
     tape.set_centered_player = FALSE;
   }
 
+  if (GameFrameDelay != GAME_FRAME_DELAY)
+    tape.property_bits |= TAPE_PROPERTY_GAME_SPEED;
+
+  if (setup.small_game_graphics || SCR_FIELDX >= 2 * SCR_FIELDX_DEFAULT)
+    tape.property_bits |= TAPE_PROPERTY_SMALL_GRAPHICS;
+
   if (!TapeAddAction(action))
     TapeStopRecording();
 }
@@ -755,6 +881,12 @@ void TapeTogglePause(boolean toggle_mode)
   if (tape.single_step && (toggle_mode & TAPE_TOGGLE_MANUAL))
     tape.single_step = FALSE;
 
+  if (tape.single_step)
+    tape.property_bits |= TAPE_PROPERTY_SINGLE_STEP;
+
+  if (tape.pausing)
+    tape.property_bits |= TAPE_PROPERTY_PAUSE_MODE;
+
   DrawVideoDisplayCurrentState();
 
   if (tape.deactivate_display)
@@ -784,12 +916,14 @@ void TapeTogglePause(boolean toggle_mode)
 
   if (game_status == GAME_MODE_PLAYING)
   {
-    if (setup.show_snapshot_buttons && CheckEngineSnapshotList())
+    if (setup.show_load_save_buttons &&
+	setup.show_undo_redo_buttons &&
+	CheckEngineSnapshotList())
     {
       if (tape.pausing)
 	MapUndoRedoButtons();
       else if (!tape.single_step)
-	UnmapUndoRedoButtons();
+	MapLoadSaveButtons();
     }
 
     ModifyPauseButtons();
@@ -851,7 +985,7 @@ byte *TapePlayAction(void)
 
   if (tape.pause_before_end)  // stop some seconds before end of tape
   {
-    if (TapeTime > tape.length_seconds - TAPE_PAUSE_SECONDS_BEFORE_DEATH)
+    if (TapeTime > (int)tape.length_seconds - TAPE_PAUSE_SECONDS_BEFORE_DEATH)
     {
       TapeStopWarpForward();
       TapeTogglePause(TAPE_TOGGLE_MANUAL);
@@ -930,6 +1064,9 @@ byte *TapePlayAction(void)
   if (tape.auto_play)
     PrintTapeReplayProgress(FALSE);
 
+  if (options.tape_log_filename != NULL)
+    WriteTapeLogfile(action);
+
   return action;
 }
 
@@ -1037,6 +1174,8 @@ void TapeQuickSave(void)
     return;
   }
 
+  tape.property_bits |= TAPE_PROPERTY_SNAPSHOT;
+
   if (SaveTapeChecked(tape.level_nr))
     SaveEngineSnapshotSingle();
 }
@@ -1095,6 +1234,7 @@ void TapeQuickLoad(void)
     TapeStartWarpForward(AUTOPLAY_MODE_WARP_NO_DISPLAY);
 
     tape.quick_resume = TRUE;
+    tape.property_bits |= TAPE_PROPERTY_SNAPSHOT;
   }
   else	// this should not happen (basically checked above)
   {
@@ -1160,12 +1300,7 @@ static boolean checkTapesFromSameLevel(struct TapeInfo *t1, struct TapeInfo *t2)
 
 static void CopyTape(struct TapeInfo *tape_from, struct TapeInfo *tape_to)
 {
-  if (tape_to->level_identifier != NULL)
-    checked_free(tape_to->level_identifier);
-
   *tape_to = *tape_from;
-
-  tape_to->level_identifier = getStringCopy(tape_from->level_identifier);
 }
 
 static void SwapTapes(struct TapeInfo *t1, struct TapeInfo *t2)
@@ -1224,16 +1359,85 @@ void FixTape_ForceSinglePlayer(void)
 // tape autoplay functions
 // ----------------------------------------------------------------------------
 
-void AutoPlayTapes(void)
+static TreeInfo *getNextValidAutoPlayEntry(TreeInfo *node)
 {
-  static LevelDirTree *autoplay_leveldir = NULL;
-  static boolean autoplay_initialized = FALSE;
-  static int autoplay_level_nr = -1;
-  static int num_levels_played = 0;
-  static int num_levels_solved = 0;
-  static int num_tapes_patched = 0;
-  static int num_tape_missing = 0;
-  static boolean level_failed[MAX_TAPES_PER_SET];
+  node = getNextValidTreeInfoEntry(node);
+
+  while (node && node->is_copy)
+    node = getNextValidTreeInfoEntry(node);
+
+  return node;
+}
+
+static TreeInfo *getFirstValidAutoPlayEntry(TreeInfo *node)
+{
+  node = getFirstValidTreeInfoEntry(node);
+
+  if (node && node->is_copy)
+    return getNextValidAutoPlayEntry(node);
+
+  return node;
+}
+
+static void AutoPlayTapes_SetScoreEntry(int score, int time)
+{
+  char *name = (options.mytapes ? setup.player_name : options.player_name);
+
+  // set unique basename for score tape (for uploading to score server)
+  strcpy(tape.score_tape_basename, getScoreTapeBasename(name));
+
+  // store score in first score entry
+  scores.last_added = 0;
+
+  struct ScoreEntry *entry = &scores.entry[scores.last_added];
+
+  strncpy(entry->tape_basename, tape.score_tape_basename, MAX_FILENAME_LEN);
+  strncpy(entry->name, setup.player_name, MAX_PLAYER_NAME_LEN);
+
+  entry->score = score;
+  entry->time = time;
+
+  PrintNoLog("- uploading score tape to score server ... ");
+
+  server_scores.uploaded = FALSE;
+}
+
+static boolean AutoPlayTapes_WaitForUpload(void)
+{
+  unsigned int upload_delay = 0;
+  unsigned int upload_delay_value = 10000;
+
+  ResetDelayCounter(&upload_delay);
+
+  // wait for score tape to be successfully uploaded (and fail on timeout)
+  while (!server_scores.uploaded)
+  {
+    if (DelayReached(&upload_delay, upload_delay_value))
+    {
+      PrintNoLog("\r");
+      Print("- uploading score tape to score server - TIMEOUT.\n");
+
+      if (program.headless)
+	Fail("cannot upload score tape to score server");
+
+      return FALSE;
+    }
+
+    UPDATE_BUSY_STATE();
+
+    Delay(20);
+  }
+
+  PrintNoLog("\r");
+  Print("- uploading score tape to score server - uploaded.\n");
+
+  return TRUE;
+}
+
+static int AutoPlayTapesExt(boolean initialize)
+{
+  static struct AutoPlayInfo autoplay;
+  static int num_tapes = 0;
   static int patch_nr = 0;
   static char *patch_name[] =
   {
@@ -1267,9 +1471,12 @@ void AutoPlayTapes(void)
 
     -1
   };
+  LevelDirTree *leveldir_current_last = leveldir_current;
+  boolean init_level_set = FALSE;
+  int level_nr_last = level_nr;
   int i;
 
-  if (autoplay_initialized)
+  if (!initialize)
   {
     if (global.autoplay_mode == AUTOPLAY_MODE_FIX)
     {
@@ -1288,7 +1495,7 @@ void AutoPlayTapes(void)
 	  SaveTapeToFilename(filename);
 
 	  tape.auto_play_level_fixed = TRUE;
-	  num_tapes_patched++;
+	  autoplay.num_tapes_patched++;
 	}
 
 	// continue with next tape
@@ -1310,67 +1517,221 @@ void AutoPlayTapes(void)
     // just finished auto-playing tape
     PrintTapeReplayProgress(TRUE);
 
+    if (options.tape_log_filename != NULL)
+      CloseTapeLogfile();
+
+    if (global.autoplay_mode == AUTOPLAY_MODE_SAVE &&
+	tape.auto_play_level_solved)
+    {
+      AutoPlayTapes_SetScoreEntry(game.score_final, game.score_time_final);
+
+      if (leveldir_current)
+      {
+	// the tape's level set identifier may differ from current level set
+	strncpy(tape.level_identifier, leveldir_current->identifier,
+		MAX_FILENAME_LEN);
+	tape.level_identifier[MAX_FILENAME_LEN] = '\0';
+
+	// the tape's level number may differ from current level number
+	tape.level_nr = level_nr;
+      }
+
+      // save score tape to upload to server; may be required for some reasons:
+      // * level set identifier in solution tapes may differ from level set
+      // * level set identifier is missing (old-style tape without INFO chunk)
+      // * solution tape may have native format (like Supaplex solution files)
+
+      SaveScoreTape(level_nr);
+      SaveServerScore(level_nr, TRUE);
+
+      AutoPlayTapes_WaitForUpload();
+    }
+
     if (patch_nr == 0)
-      num_levels_played++;
+      autoplay.num_levels_played++;
 
     if (tape.auto_play_level_solved)
-      num_levels_solved++;
+      autoplay.num_levels_solved++;
 
     if (level_nr >= 0 && level_nr < MAX_TAPES_PER_SET)
-      level_failed[level_nr] = !tape.auto_play_level_solved;
+      autoplay.level_failed[level_nr] = !tape.auto_play_level_solved;
   }
   else
   {
-    DrawCompleteVideoDisplay();
+    if (strEqual(global.autoplay_leveldir, "ALL"))
+    {
+      autoplay.all_levelsets = TRUE;
 
-    audio.sound_enabled = FALSE;
-    setup.engine_snapshot_mode = getStringCopy(STR_SNAPSHOT_MODE_OFF);
+      // tape mass-uploading only allowed for private tapes
+      if (global.autoplay_mode == AUTOPLAY_MODE_UPLOAD)
+	options.mytapes = TRUE;
+    }
 
-    autoplay_leveldir = getTreeInfoFromIdentifier(leveldir_first,
-						  global.autoplay_leveldir);
+    if ((global.autoplay_mode == AUTOPLAY_MODE_SAVE ||
+	 global.autoplay_mode == AUTOPLAY_MODE_UPLOAD) &&
+	!options.mytapes &&
+	options.player_name == NULL)
+    {
+      Fail("specify player name when uploading solution tapes");
+    }
 
-    if (autoplay_leveldir == NULL)
-      Fail("no such level identifier: '%s'", global.autoplay_leveldir);
+    if (global.autoplay_mode != AUTOPLAY_MODE_UPLOAD)
+      DrawCompleteVideoDisplay();
 
-    leveldir_current = autoplay_leveldir;
+    if (program.headless)
+    {
+      audio.sound_enabled = FALSE;
+      setup.engine_snapshot_mode = getStringCopy(STR_SNAPSHOT_MODE_OFF);
+    }
 
-    if (autoplay_leveldir->first_level < 0)
-      autoplay_leveldir->first_level = 0;
-    if (autoplay_leveldir->last_level >= MAX_TAPES_PER_SET)
-      autoplay_leveldir->last_level = MAX_TAPES_PER_SET - 1;
+    if (strSuffix(global.autoplay_leveldir, ".tape"))
+    {
+      autoplay.tape_filename = global.autoplay_leveldir;
 
-    autoplay_level_nr = autoplay_leveldir->first_level;
+      if (!fileExists(autoplay.tape_filename))
+	Fail("tape file '%s' does not exist", autoplay.tape_filename);
 
-    PrintLine("=", 79);
-    if (global.autoplay_mode == AUTOPLAY_MODE_FIX)
-      Print("Automatically fixing level tapes\n");
+      LoadTapeFromFilename(autoplay.tape_filename);
+
+      if (tape.no_valid_file)
+	Fail("cannot load tape file '%s'", autoplay.tape_filename);
+
+      if (tape.no_info_chunk && !options.identifier)
+	Fail("cannot get levelset from tape file '%s'", autoplay.tape_filename);
+
+      if (tape.no_info_chunk && !options.level_nr)
+	Fail("cannot get level nr from tape file '%s'", autoplay.tape_filename);
+
+      global.autoplay_leveldir = tape.level_identifier;
+
+      if (options.identifier != NULL)
+	global.autoplay_leveldir = options.identifier;
+
+      if (options.level_nr != NULL)
+	tape.level_nr = atoi(options.level_nr);
+
+      if (tape.level_nr >= 0 && tape.level_nr < MAX_TAPES_PER_SET)
+        global.autoplay_level[tape.level_nr] = TRUE;
+
+      global.autoplay_all = FALSE;
+      options.mytapes = FALSE;
+    }
+
+    if (autoplay.all_levelsets)
+    {
+      // start auto-playing first level set
+      autoplay.leveldir = getFirstValidAutoPlayEntry(leveldir_first);
+    }
     else
-      Print("Automatically playing level tapes\n");
-    PrintLine("-", 79);
-    Print("Level series identifier: '%s'\n", autoplay_leveldir->identifier);
-    Print("Level series name:       '%s'\n", autoplay_leveldir->name);
-    Print("Level series author:     '%s'\n", autoplay_leveldir->author);
-    Print("Number of levels:        %d\n",   autoplay_leveldir->levels);
-    PrintLine("=", 79);
-    Print("\n");
+    {
+      // auto-play selected level set
+      autoplay.leveldir = getTreeInfoFromIdentifier(leveldir_first,
+						    global.autoplay_leveldir);
+    }
 
-    for (i = 0; i < MAX_TAPES_PER_SET; i++)
-      level_failed[i] = FALSE;
+    if (autoplay.leveldir == NULL)
+      Fail("no such level identifier: '%s'", global.autoplay_leveldir);
 
     // only private tapes may be modified
     if (global.autoplay_mode == AUTOPLAY_MODE_FIX)
       options.mytapes = TRUE;
 
-    autoplay_initialized = TRUE;
+    // set timestamp for batch tape upload
+    global.autoplay_time = time(NULL);
+
+    num_tapes = 0;
+
+    init_level_set = TRUE;
   }
 
   while (1)
   {
-    if (global.autoplay_mode != AUTOPLAY_MODE_FIX || patch_nr == 0)
-      level_nr = autoplay_level_nr++;
+    if (init_level_set)
+    {
+      leveldir_current = autoplay.leveldir;
 
-    if (level_nr > autoplay_leveldir->last_level)
-      break;
+      if (autoplay.leveldir->first_level < 0)
+	autoplay.leveldir->first_level = 0;
+      if (autoplay.leveldir->last_level >= MAX_TAPES_PER_SET)
+	autoplay.leveldir->last_level = MAX_TAPES_PER_SET - 1;
+
+      autoplay.level_nr = autoplay.leveldir->first_level;
+
+      autoplay.num_levels_played = 0;
+      autoplay.num_levels_solved = 0;
+      autoplay.num_tapes_patched = 0;
+      autoplay.num_tape_missing = 0;
+
+      for (i = 0; i < MAX_TAPES_PER_SET; i++)
+	autoplay.level_failed[i] = FALSE;
+
+      PrintTapeReplayHeader(&autoplay);
+
+      init_level_set = FALSE;
+    }
+
+    if (autoplay.all_levelsets && global.autoplay_mode == AUTOPLAY_MODE_UPLOAD)
+    {
+      boolean skip_levelset = FALSE;
+
+      if (!directoryExists(getTapeDir(autoplay.leveldir->subdir)))
+      {
+	Print("No tape directory for this level set found -- skipping.\n");
+
+	skip_levelset = TRUE;
+      }
+
+      if (CheckTapeDirectoryUploadsComplete(autoplay.leveldir->subdir))
+      {
+	Print("All tapes for this level set already uploaded -- skipping.\n");
+
+	skip_levelset = TRUE;
+      }
+
+      if (skip_levelset)
+      {
+	PrintTapeReplaySummary(&autoplay);
+
+	// continue with next level set
+	autoplay.leveldir = getNextValidAutoPlayEntry(autoplay.leveldir);
+
+	// all level sets processed
+	if (autoplay.leveldir == NULL)
+	  break;
+
+	init_level_set = TRUE;
+
+	continue;
+      }
+    }
+
+    if (global.autoplay_mode != AUTOPLAY_MODE_FIX || patch_nr == 0)
+      level_nr = autoplay.level_nr++;
+
+    UPDATE_BUSY_STATE();
+
+    // check if all tapes for this level set have been processed
+    if (level_nr > autoplay.leveldir->last_level)
+    {
+      PrintTapeReplaySummary(&autoplay);
+
+      if (!autoplay.all_levelsets)
+	break;
+
+      if (global.autoplay_mode == AUTOPLAY_MODE_UPLOAD)
+	MarkTapeDirectoryUploadsAsComplete(autoplay.leveldir->subdir);
+
+      // continue with next level set
+      autoplay.leveldir = getNextValidAutoPlayEntry(autoplay.leveldir);
+
+      // all level sets processed
+      if (autoplay.leveldir == NULL)
+	break;
+
+      init_level_set = TRUE;
+
+      continue;
+    }
 
     // set patch info (required for progress output)
     strcpy(tape_patch_info, "");
@@ -1381,7 +1742,18 @@ void AutoPlayTapes(void)
     if (!global.autoplay_all && !global.autoplay_level[level_nr])
       continue;
 
+    // speed things up in case of missing private tapes (skip loading level)
+    if (options.mytapes && !fileExists(getTapeFilename(level_nr)))
+    {
+      autoplay.num_tape_missing++;
+
+      Print("Tape %03d: (no tape found)\n", level_nr);
+
+      continue;
+    }
+
     TapeErase();
+    TapeRewind();	// needed to reset "tape.auto_play_level_solved"
 
     LoadLevel(level_nr);
 
@@ -1398,14 +1770,16 @@ void AutoPlayTapes(void)
     continue;
 #endif
 
-    if (options.mytapes)
+    if (autoplay.tape_filename)
+      LoadTapeFromFilename(autoplay.tape_filename);
+    else if (options.mytapes)
       LoadTape(level_nr);
     else
       LoadSolutionTape(level_nr);
 
     if (tape.no_valid_file)
     {
-      num_tape_missing++;
+      autoplay.num_tape_missing++;
 
       Print("Tape %03d: (no tape found)\n", level_nr);
 
@@ -1469,40 +1843,100 @@ void AutoPlayTapes(void)
       }
     }
 
+    num_tapes++;
+
+    if (global.autoplay_mode == AUTOPLAY_MODE_UPLOAD)
+    {
+      boolean use_temporary_tape_file = FALSE;
+
+      Print("Tape %03d:\n", level_nr);
+
+      AutoPlayTapes_SetScoreEntry(0, 0);
+
+      if (autoplay.tape_filename == NULL)
+      {
+	autoplay.tape_filename = (options.mytapes ? getTapeFilename(level_nr) :
+				  getDefaultSolutionTapeFilename(level_nr));
+
+	if (!fileExists(autoplay.tape_filename))
+	{
+	  // non-standard or incorrect solution tape -- save to temporary file
+	  autoplay.tape_filename = getTemporaryTapeFilename();
+
+	  SaveTapeToFilename(autoplay.tape_filename);
+
+	  use_temporary_tape_file = TRUE;
+	}
+      }
+
+      SaveServerScoreFromFile(level_nr, TRUE, autoplay.tape_filename);
+
+      boolean success = AutoPlayTapes_WaitForUpload();
+
+      if (use_temporary_tape_file)
+        unlink(autoplay.tape_filename);
+
+      // required for uploading multiple tapes
+      autoplay.tape_filename = NULL;
+
+      if (!success)
+      {
+	num_tapes = -num_tapes;
+
+	break;
+      }
+
+      continue;
+    }
+
     InitCounter();
 
+    if (options.tape_log_filename != NULL)
+      OpenTapeLogfile();
+
     TapeStartGamePlaying();
     TapeStartWarpForward(global.autoplay_mode);
 
-    return;
+    autoplay.last_level_nr = level_nr;
+
+    return num_tapes;
   }
 
-  Print("\n");
-  PrintLine("=", 79);
-  Print("Number of levels played: %d\n", num_levels_played);
-  Print("Number of levels solved: %d (%d%%)\n", num_levels_solved,
-	(num_levels_played ? num_levels_solved * 100 / num_levels_played : 0));
-  if (global.autoplay_mode == AUTOPLAY_MODE_FIX)
-    Print("Number of tapes fixed: %d\n", num_tapes_patched);
-  PrintLine("-", 79);
-  Print("Summary (for automatic parsing by scripts):\n");
-  Print("LEVELDIR [%s] '%s', SOLVED %d/%d (%d%%)",
-	(num_levels_played == num_levels_solved ? " OK " : "WARN"),
-	autoplay_leveldir->identifier, num_levels_solved, num_levels_played,
-	(num_levels_played ? num_levels_solved * 100 / num_levels_played : 0));
-
-  if (num_levels_played != num_levels_solved)
+  if (global.autoplay_mode == AUTOPLAY_MODE_UPLOAD)
   {
-    Print(", FAILED:");
-    for (i = 0; i < MAX_TAPES_PER_SET; i++)
-      if (level_failed[i])
-	Print(" %03d", i);
+    Print("\n");
+    PrintLine("=", 79);
+
+    if (num_tapes >= 0)
+      Print("SUMMARY: %d tapes uploaded.\n", num_tapes);
+    else
+      Print("SUMMARY: Uploading tapes failed.\n");
+
+    PrintLine("=", 79);
   }
 
-  Print("\n");
-  PrintLine("=", 79);
+  // clear timestamp for batch tape upload (required after interactive upload)
+  global.autoplay_time = 0;
 
-  CloseAllAndExit(0);
+  // exit if running headless or if visually auto-playing tapes
+  if (program.headless || global.autoplay_mode != AUTOPLAY_MODE_UPLOAD)
+    CloseAllAndExit(0);
+
+  // when running interactively, restore last selected level set and number
+  leveldir_current = leveldir_current_last;
+  level_nr = level_nr_last;
+
+  return num_tapes;
+}
+
+int AutoPlayTapes(void)
+{
+  return AutoPlayTapesExt(TRUE);
+}
+
+int AutoPlayTapesContinue(void)
+{
+  return AutoPlayTapesExt(FALSE);
 }
 
 
